#include "mpi.h"
#include "mpiimpl.h"

/*======================================================================*/

int MPI_Recv(void *buf, int count, MPI_Datatype datatype, 
            int source, int tag, MPI_Comm comm, MPI_Status *status ) {
    MPI__INT d; /* dummy */

    if (count < 0 || datatype < 1 || datatype > MPI_LONG_LONG_INT ||
        comm == MPI_COMM_NULL ||
        MPI_Comms[comm].group->nodes[source] == MPI_UNDEFINED) {
      printf("MPI_Recv(.., %d, %d, %d, %d, %d,..): invalid argument\n", 
	     count, datatype, source, tag, comm);
      exit(1);
    }    
    return MPI_internal_rcv(comm, source, tag, buf, count * MPI_datatype_size[datatype], 
			    datatype, &d, &d, &d, status);
}

/*======================================================================*/
/* Standard send behaves as a buffered if there are left buffer space
   (at receivers side & socket buffer) and after that as a synchronuous */
   
int MPI_Send(void *buf, int count, MPI_Datatype datatype,  
            int dest, int tag, MPI_Comm comm) {
    MPI__INT d = 0; /* dummy */

    if (count < 0 || datatype < 1 || datatype > MPI_LONG_LONG_INT ||
        comm == MPI_COMM_NULL ||
        MPI_Comms[comm].group->nodes[dest] == MPI_UNDEFINED) {
        printf("MPI_Send(.., %d, %d, %d, %d, %d): invalid argument\n", 
	       count, datatype, dest, tag, comm);
        exit(1);
    }    
    return MPI_internal_snd(comm, dest, tag, buf, count * MPI_datatype_size[datatype], 
			    datatype, d, d, d);
}                

/*======================================================================*/

/* MUST SET STATUS FOR TWO PROBE COMMANDS */
int MPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status) {
    int d;
    struct MPI_msg_hdr hdr;
    MPI_read_msg_hdr(source, tag, comm, &d, &hdr, &d, MPI_BLOCKING, MPI_DONT_EAT_HDR);
    status->MPI_TAG = hdr.tag;
    status->MPI_SOURCE = hdr.source;
    status->mpi_size = hdr.size;
    return MPI_SUCCESS;
}

/*======================================================================*/

int MPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status) {
    struct MPI_msg_hdr hdr;
    int d;

    MPI_read_msg_hdr(source, tag, comm, flag, &hdr, &d,
		     MPI_NOT_BLOCKING, MPI_DONT_EAT_HDR);    
    status->MPI_TAG = hdr.tag;
    status->MPI_SOURCE = hdr.source;
    status->mpi_size = hdr.size;
    return MPI_SUCCESS;
}

/*======================================================================*/
