#include "mpi.h"
#include "mpiimpl.h"

#include <string.h>

/*======================================================================*/

int MPI_Get_count(MPI_Status *status, MPI_Datatype datatype, int *count) {
    *count = status->mpi_size / MPI_datatype_size[datatype];

#ifdef DEBUG
    printf("MPI_Get_count:  count: %d\n", *count);
#endif
  return MPI_SUCCESS;    
}
/*========================================================================================*/

int MPI_Get_processor_name(char *name, int *resultlen) {
    if (MPI_processor_name_len < *resultlen)
        *resultlen = MPI_processor_name_len;    
    memcpy(name, MPI_processor_name, *resultlen);
    return MPI_SUCCESS;
}

/*========================================================================================*/

int MPI_Get_version(int *version, int *subversion) {
    *version = MPI_VERSION;
    *subversion = MPI_SUBVERSION;
    return MPI_SUCCESS;
}

/*========================================================================================*/

int MPI_Error_string(int errorcode, char *string, int *resultlen) {
    *resultlen = 0; /* not supported */
    return MPI_SUCCESS;    
}

/*========================================================================================*/
/* support onlu predifined attributes */
int MPI_Attr_get (MPI_Comm comm, int keyval, void *attr_value, int *flag) {
    struct MPI_Comm_entry entry;
  if (comm == MPI_COMM_NULL) {
    *flag = 0;
    return MPI_ERR_COMM;
  }

    entry = MPI_Comms[comm];        
    *flag = 1;
    switch(keyval) {
        case MPI_TAG_UB: 
            *(int *)attr_value = ~(1 << (8 * sizeof(MPI__INT) - 1) );
            break;
        case MPI_HOST:
            if (entry.group->ranks[0] != MPI_UNDEFINED)
                *(int *)attr_value = entry.group->ranks[0];
            else 
                *(int *)attr_value = MPI_PROC_NULL;
            break;
        case MPI_IO:
            *(int *)attr_value = entry.source;
            break;
        case MPI_WTIME_IS_GLOBAL: 
            *(int *)attr_value = 0;
            break;
        default: 
            *flag = 0;
    }
    return  MPI_SUCCESS;
}

/*========================================================================================*/
int MPI_Comm_get_parent(MPI_Comm *parent) {
  *parent = MPI_COMM_PARENT;
  return MPI_SUCCESS;
}

/*========================================================================================*/
/*========================================================================================*/
/*========================================================================================*/

