#include "mpi.h"
#include "mpiimpl.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/stat.h> 

/*========================================================================================*/
/* last three arguments are 
   master_host_name, master_port and slave_tag;
   slave_tag beeing the last */   

int MPI_slave(int *argc_ptr, char ***argv_ptr) {

  char *master_host;  /* master host */
  char *port;         /* port to connect to */
  
  int sd;                    /* socket descriptor       */
  struct MPI_init_msg msg1;
  struct hostent *mhp;       /* ptr to host info structure*/
  struct sockaddr_in sin, list_addr; /* inet info structures */
  int i, size;

  /* holders for type sizes that can be pull out of the loops */
  size_t msg1_size = sizeof(struct MPI_init_msg);
  size_t addr_size = sizeof(struct sockaddr_in);

  MPI_Group group;
  struct MPI_Comm_entry *entry;
    
  /* get extra parameters and hide them from user */
  master_host = (*argv_ptr)[*argc_ptr - 3];
  port = (*argv_ptr)[*argc_ptr - 2];

  (*argv_ptr)[*argc_ptr - 3] = NULL;
  *argc_ptr -= 3;

#ifdef DEBUG
  printf("Slave's hostname is: %s\n", MPI_processor_name);
#endif
            
  MPI_new_listener(&MPI_my_list_sd, &list_addr, &MPI_my_port);/* slave sets up listener */

#ifdef DEBUG
  printf("Slave now listens at port %d with socket %d\n", MPI_my_port, MPI_my_list_sd);
#endif
  
  mhp = gethostbyname(master_host); /* get the host info such as IP address */
  
  bzero((char*)&sin, addr_size);
  memcpy((char *)&sin.sin_addr, mhp->h_addr, mhp->h_length); /* copy the IP address      */  
  sin.sin_port = htons(atoi(port));                          /* set port ID              */  
  sin.sin_family = mhp->h_addrtype;                          /* sand pecify address type */

  /* we are using the default protocol 6 = IPPROTO_TCP */
  CALL_CHK(sd = socket, (AF_INET, SOCK_STREAM, IPPROTO_TCP));
  /* Connect to master and set its socket desccr. into procgroup array */
  CALL_CHK(connect, (sd, (struct sockaddr*)&sin, addr_size));

  /* this is socket for communication with master process */  
  MPI_nodes[0].sd = sd;
  MPI_new_sd(sd);

  CALL_CHK(send, (sd, (char *)&list_addr, addr_size, 0)); /* send my listener address */

  CALL_CHK(size = recv, (sd, (char *)&msg1, msg1_size, 0)); /* receive initial message */
  if (size != msg1_size) {
    printf("Incomplete message received.\n");
    exit(1);
  }
  if (ntohl(msg1.len) != msg1_size) {
    printf("Size of struct or int not preserved across architectures.\n");
    printf("Either contact maintainer or use homogeneous architecture.\n");
    exit(1);
  }
  MPI_node_num = ntohl(msg1.node_num);
  MPI_num_nodes = ntohl(msg1.num_nodes);

  /* Set listener_addr field of pg_array field on slave */  
  for (i = 0; i < MPI_num_nodes; i++ ) {
    CALL_CHK(size = recv, (sd, (char *)&(MPI_nodes[i].listener_addr), addr_size, 0));
    if (size != addr_size) {
      printf("Incomplete message received.\n");
      exit(1);
    }
  }

  { 
    MPI__INT buf;
    /* Acknowledge that master and slave are synchronized */
    CALL_CHK( recv, (sd, (char *)&buf, sizeof(buf), 0) );
    if (ntohl(buf) != addr_size) {
      printf("MPI_Init: slave %d not synchronized.\n", MPI_node_num);
      exit(1);
    }
  }

  MPI_createSelfComm();
  MPI_createWorldComm(MPI_num_nodes);

#ifdef DEBUG
  printf("Slave node %d of %d initialized\n", MPI_node_num, MPI_nodes);
#endif

  return MPI_SUCCESS;
}


