#include "mpi.h"

#include "mpiimpl.h"


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/stat.h> 

/*========================================================================================*/
/* last three arguments are 
   parent_host_name, parent_port and spawned_tag;
   slave_tag beeing the last */   

int MPI_spawned(int *argc_ptr, char ***argv_ptr) {

  char *parent_host;  /* parent host        */
  char *port;         /* port to connect to */
  
  int sd;                 /* socket descriptor       */
  struct MPI_init_msg1 msg1;
  struct hostent *php;  /* ptr to host info structure*/
  struct sockaddr_in sin, list_addr; /* inet info structures */
  int i, size;
  int group_size, my_rank, remote_group_size;

  /* holders for type sizes that can be pull out of the loops */
  size_t msg1_size = sizeof(struct MPI_init_msg1);
  size_t addr_size = sizeof(struct sockaddr_in);

  MPI_Group group;
  struct MPI_Comm_entry *entry, *parent_entry;
    
  /* get extra parameters and hide them from user */
  parent_host = (*argv_ptr)[*argc_ptr - 3];
  port = (*argv_ptr)[*argc_ptr - 2];

  (*argv_ptr)[*argc_ptr - 3] = NULL;
  *argc_ptr -= 3;

#ifdef DEBUG
  printf("Spawned  hostname is: %s\n", MPI_processor_name);
#endif

  MPI_new_listener(&MPI_my_list_sd, &list_addr, &MPI_my_port);/* sets up listener */

#ifdef DEBUG
  printf("Spawned node now listens at port %d with socket %d\n", 
     MPI_my_port, MPI_my_list_sd);
#endif

  php = gethostbyname(parent_host); /* get the host info such as IP address */
  
  bzero((char*)&sin, addr_size);
  memcpy((char *)&sin.sin_addr, php->h_addr, php->h_length); /* copy the IP address      */  
  sin.sin_port = htons(atoi(port));                          /* set port ID              */  
  sin.sin_family = php->h_addrtype;                          /* sand pecify address type */

  /* we are using the default protocol 6 = IPPROTO_TCP */
  CALL_CHK(sd = socket, (AF_INET, SOCK_STREAM, IPPROTO_TCP));
  /* Connect to parent */
  CALL_CHK(connect, (sd, (struct sockaddr*)&sin, addr_size));

  CALL_CHK(send, (sd, (char *)&list_addr, addr_size, 0));/* send my listener address */

  CALL_CHK(size = recv, (sd, (char *)&msg1, msg1_size, 0));/* receive initial message */
  if (size != msg1_size) {
    printf("Incomplete message received.\n");
    exit(1);
  }
  if (ntohl(msg1.len) != msg1_size) {
    printf("Size of struct or int not preserved across architectures.\n");
    printf("Either contact maintainer or use homogeneous architecture.\n");
    exit(1);
  }

  MPI_node_num = my_rank = ntohl(msg1.rank);
  remote_group_size = ntohl(msg1.spawned_nodes);
  group_size = ntohl(msg1.num_nodes);
  MPI_num_nodes = group_size + remote_group_size;

#ifdef DEBUG
  printf("spawn rank %d, node number %d, nodes %d\n", 
     my_rank, MPI_node_num, MPI_num_nodes);
#endif

  /* Set listener_addr field of MPI_nodes on spawned process */  
  for (i = 0; i < MPI_num_nodes; i++) {
    CALL_CHK(size = recv, (sd, (char *)&(MPI_nodes[i].listener_addr), addr_size, 0));
    if (size != addr_size) {
      printf("Incomplete message received.\n");
      exit(1);
    }
  }

  { 
    MPI__INT buf;
    /* Acknowledge that parent and spawned node are synchronized */
    CALL_CHK( recv, (sd, (char *)&buf, sizeof(buf), 0) );
    if (ntohl(buf) != addr_size) {
      printf("MPI_Init: slave %d not synchronized.\n", MPI_node_num);
      exit(1);
    }
#ifdef DEBUG
    printf("MPI_Init: slave %d synchronized.\n", MPI_node_num);
#endif
  }

  /* create MPI_COMM_WORLD */
  MPI_createWorldComm(group_size);

  /* create inter communicator (parent comm) */
  {
    int i;

    MPI_COMM_PARENT = MPI_first_available_comm++;
    parent_entry = &MPI_Comms[MPI_COMM_PARENT];

   /* COMM_WORLD is a local group */
    parent_entry->local = MPI_Comms[MPI_COMM_WORLD].group; 
    parent_entry->local->references++;

    group = MPI_getGroup();
    for (i = 0; i < remote_group_size; i++) {
      group->nodes[i] = group_size + i;
      group->ranks[group_size +i] = i;
    }
    group->size = remote_group_size;
    parent_entry->group = group;
    group->references++;
    MPI_Group_free(&group);
    

    parent_entry->snd_context = ntohl(msg1.send_context);
    parent_entry->rcv_context = MPI_COMM_PARENT;
    parent_entry->intracomm = 0;
    parent_entry->source = my_rank;
    parent_entry->used = 1;
    parent_entry->tag = 0;

#ifdef DEBUG
    printf("parent send context is %d\n", parent_entry->snd_context);
#endif 
  }

  MPI_createSelfComm();
  return MPI_SUCCESS;
}
