#include "mpiimpl.h"

#include <sys/socket.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#define RCVBUF_LEN 1000
#define SNDBUF_LEN 1000

/*========================================================================================*/

void MPI_new_sd(int sd) {
  int i;
  setsockopt(sd, SOL_SOCKET, SO_KEEPALIVE, (void *)&i, sizeof(i));

  FD_SET(sd, &MPI_fdset);
  if (sd > MPI_max_sd )
    MPI_max_sd = sd;
}

static size_t len = sizeof(struct sockaddr_in);
static struct sockaddr sin;

/*
#define MPI_is_connected(sd) (1 + getpeername((sd), &sin, &len))
*/
int MPI_is_connected(int sd) { 
  return (1 + getpeername(sd, &sin, &len));
}


/*========================================================================================*/

void MPI_pack_group(MPI_Group group, char *buff) {
  int i;
  size_t step_size = sizeof(MPI__INT);  
  
  for (i = 0; i < MPI_MAX_NODES; i++) {
    *(MPI__INT *)buff = htonl(group->nodes[i]);
    buff = buff + step_size;
  }
  for (i = 0; i < MPI_MAX_NODES; i++) {
    *(MPI__INT *)buff = htonl(group->ranks[i]);
    buff = buff + step_size;
  }
  *(MPI__INT *)buff = htonl(group->size);
}

/*========================================================================================*/

void MPI_unpack_group(char *buff, MPI_Group *group) {
  size_t step_size = sizeof(MPI__INT);
  int i;
    
  for (i = 0; i < MPI_MAX_NODES; i++) {
    (*group)->nodes[i] = ntohl(*(MPI__INT *)buff);
    buff = buff + step_size;
  }

  for (i = 0; i < MPI_MAX_NODES; i++) {
    (*group)->ranks[i] = ntohl(*(MPI__INT *)buff);
    buff = buff + step_size;
  }
  (*group)->size = ntohl(*(MPI__INT *)buff);
}

/*========================================================================================*/
/* fork and run process from it */
#define RSH "rsh"
#define RSH_ENV "RSH"

/* Tag can be either SLAVE_TAG or SPAWNED tag.
   Hostname and port are needed for forked process to start communicaton */
pid_t MPI_fork_process(char *processor, char *process, 
              int argc, char *argv[],
              char *hostname, char *port, char *tag) {
  char *rsh_cmd;
  int i, fd;
  pid_t pid;
  char args[256];

  /* collect arguments */
  args[0] = '\0';
  for (i = 0; i < argc; i++) 
    sprintf(args + strlen(args), "%s ", argv[i]);
  
  if ((rsh_cmd = getenv(RSH_ENV)) == NULL ) 
    rsh_cmd = RSH;

#ifdef DEBUG  
if ((strcmp(processor, "localhost") == 0) ||
        (strcmp(processor, MPI_processor_name) == 0)) 
  printf("Exec: %s %s %s %s %s\n", process , args, hostname, port, tag);
else 
  printf("Exec: %s %s exec %s %s %s %s %s\n",
     		rsh_cmd, processor, process , args, hostname, port, tag);
#endif
  
  CALL_CHK(pid = fork,());
  if (pid == (pid_t)0) {  /* if child process */

#ifndef STDIN_FILENO
#define STDIN_FILENO 0 /* Should be standard in most dialects -- just in case */
#endif
    /* close input of the child process */
    close(STDIN_FILENO);
    CALL_CHK(fd = open, ("/dev/null", O_RDONLY));
    if (fd != STDIN_FILENO) { /* fd should be stdin, but check to be safe */
      CALL_CHK(dup2, (fd, STDIN_FILENO));
      close(fd);
    }
    if ((strcmp(processor, "localhost") == 0) ||
        (strcmp(processor, MPI_processor_name) == 0)) 
     /* local execute */    
      i = execlp( process, args, hostname, 
		port, tag, (char*)NULL);
    else  /*remote execute */
      i = execlp(rsh_cmd, processor, "exec" , 
                 process, args, hostname, port, tag, (char*)NULL);
    exit(i); /* child exits after slave finishes */
  }
  /* master process just continues */
  return pid;
}

/*========================================================================================*/

