#include "mpi.h"
#include "mpiimpl.h"

#include <stdio.h>

void master(), slave(), spawned(), not_spawned();

int myrank; 
MPI_Comm intercomm, parent, merged_intracomm;

main(int argc,char *argv[]) {
  printf("HELLO3-MPI\n");
  
  if (MPI_Init(&argc, &argv) != MPI_SUCCESS){       /* initialize MPI */
    printf("MPI_Init error\n");
    exit(1);
  }
  
  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
  printf("MPI_Init done on %d\n", myrank);
 
  MPI_Comm_get_parent(&parent);

  if (parent == MPI_COMM_NULL) {
	printf("I am not spawned\n");
	not_spawned();
  }
  else {
        printf("I am spawned\n");
	spawned();
  }
  MPI_Finalize();           /* cleanup MPI */
  printf("MPI_Exit done on %d comm %d\n", myrank, MPI_COMM_WORLD);
  exit(0);
}

void not_spawned() {
  int proc = 2;
 
  MPI_Comm_spawn("/home/predrag/com3200/tests/hello3", 
		MPI_ARGV_NULL, proc, MPI_INFO_NULL, 
		0, MPI_COMM_WORLD, &intercomm, MPI_ERRCODES_IGNORE);

  if (myrank == 0) 
    master();
  else 
    slave();
}

void master() {
  MPI_Status    status;
  int s,d;
  
  printf("M A S T E R\n");

  MPI_Comm_size(MPI_COMM_WORLD, &d);
  printf("World %d size = %d\n",MPI_COMM_WORLD, d);
  
  printf("Intercomm number is = %d, snd_context is %d, rcv_context = %d\n", 
    intercomm, MPI_Comms[intercomm].snd_context, MPI_Comms[intercomm].rcv_context);

  MPI_Comm_remote_size(intercomm, &d);
  printf("Remote %d group size = %d\n", intercomm,d);
  
  MPI_Comm_size(intercomm, &d);
  printf("Local group size = %d\n", d);
  
  MPI_Recv(&d, 1, MPI_INT, 0, 66, intercomm, &status);  
  printf("Received from spawned root %d\n", d); 
  
  d = 48;
  MPI_Send(&d, 1, MPI_INT, 0, 55, intercomm);
  printf("Sent to spawned root %d\n", d);   

}

void slave() {
  MPI_Status    status;
  int s, d;
  printf("S L A V E\n");
}

void spawned_master() {
  int d;
  MPI_Status status;

  printf("SPAWNED M A S T E R\n");
  
  d = 111;
  MPI_Send(&d, 1, MPI_INT, 0, 66, parent);  

  MPI_Recv(&d, 1, MPI_INT, 0, 55, parent, &status);
  printf("Receive from root %d\n", d);   

}

void spawned_slave() {
  MPI_Status    status;
  int s, d;
  printf("SPAWNED S L A V E\n");
}

void spawned() {
  if (myrank == 0) 
    spawned_master();
  else 
    spawned_slave();
}
