#include "mpi.h"
#include "mpiimpl.h"
#include <stdio.h>

void group0root(void);
void group1root(void);
void group0(void);
void group1(void);


/* Two-Group "Pipeline" example */
MPI_Comm myComm;        /* intra-communicator of local sub-group */
MPI_Comm myFirstComm;   /* inter-communicator */
int membershipKey;
int rank, grouprank;

main(int argc,char *argv[]) {
    int i, j;
    
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    
    /* User code must generate membershipKey in the range [0, 1] */
    membershipKey = rank % 2;
    
    /* Build intra-communicator for local sub-group */
    MPI_Comm_split(MPI_COMM_WORLD, membershipKey, rank, &myComm);
/*
    printf("Comm world split at two groups\n");

    printf("Size of %d is %d at node %d\n",
    myComm, MPI_Comms[myComm].group->size, MPI_node_num);
    
    for (i = 0; i < MPI_Comms[myComm].group->size; i++) {
        j = MPI_Comms[myComm].group->nodes[i];
        printf("(rank, node) = (%d, %d) in myComm = %d at node %d\n",
         i, j, myComm, MPI_node_num);
    }    
*/
    /* Build inter-communicator. Tags are hard-coded */

    switch (membershipKey) {
        case 0: /* Group 0 communicate with group 1 */
                MPI_Intercomm_create(myComm, 0, MPI_COMM_WORLD, 1, 
                                17, &myFirstComm);
  /*              printf("Group 0-1 intercomm created\n");*/
                break;
        case 1: /* Group 1 communicate with groups 0 and 2 */
                MPI_Intercomm_create(myComm, 0, MPI_COMM_WORLD, 0, 
                                17, &myFirstComm);
  /*              printf("Group 1-0 intercomm created\n");*/
                break;
        default:printf("ERRROR\n"); 
		break;
    }

/*    printf("All intercomms created\n");*/

/* Do work .... */
    MPI_Comm_rank(myComm, &grouprank);
/*    if (grouprank == 0) /* root of local group *
        switch (membershipKey) {
            case 0: group0root(); break;
            case 1: group1root(); break;
            default: printf("ERROR2\n"); break;
        }
*/   
    
    switch (membershipKey) {
	case 0: group0(); break;
	case 1: group1(); break;
	default: printf("ERROR3\n"); break;
    }

    MPI_Finalize();
    exit(0);
}

void group0root(void) {
    int i;

    for (i = 0;  i < MPI_Comms[myFirstComm].group->size; i++) {
	int node = MPI_Comms[myFirstComm].group->nodes[i];
	printf("Remote group(%d) (node, rank) = (%d, %d)\n", rank, node, i);
   }
    for (i = 0;  i < MPI_Comms[myFirstComm].local->size; i++) {
	int node = MPI_Comms[myFirstComm].local->nodes[i];
	printf("Local group(%d) (node, rank) = (%d, %d)\n", rank, node, i);
   }
}

void group1root(void){
    int i;
    for (i = 0;  i < MPI_Comms[myFirstComm].group->size; i++) {
	int node = MPI_Comms[myFirstComm].group->nodes[i];
	printf("Remote group(%d) (node, rank) = (%d, %d)\n", rank, node, i);
   }
    for (i = 0;  i < MPI_Comms[myFirstComm].local->size; i++) {
	int node = MPI_Comms[myFirstComm].local->nodes[i];
	printf("Local group(%d) (node, rank) = (%d, %d)\n", rank, node, i);
   }
}


void group0(void) {
   int i, data;
   int remote_size, lrank, local_size;
   MPI_Status status;

   MPI_Comm_rank(myFirstComm, &lrank);
   MPI_Comm_size(myFirstComm, &local_size);
   MPI_Comm_remote_size(myFirstComm, &remote_size);

   for (i = 0; i < remote_size; i++) {
	data = rank;
	MPI_Send(&data, 1, MPI_INT, i, 0, myFirstComm);
   }

   for (i = 0; i < remote_size; i++) {
	MPI_Recv(&data, 1, MPI_INT, i, 0, myFirstComm, &status);
	/*if (lrank == 0)*/
	printf("Recevied(%d) from %d data %d \n", rank, i, data);
   }
}

void group1(void){
   int i, data;
   int remote_size, lrank, local_size;
   MPI_Status status;

   MPI_Comm_rank(myFirstComm, &lrank);
   MPI_Comm_size(myFirstComm, &local_size);
   MPI_Comm_remote_size(myFirstComm, &remote_size);

   for (i = 0; i < remote_size; i++) {
	MPI_Recv(&data, 1, MPI_INT, i, 0, myFirstComm, &status);
	/*if (lrank == 0)*/
	printf("Recevied(%d) from %d data %d \n", rank, i, data);
   }

   for (i = 0; i < remote_size; i++) {
	data = rank;
	MPI_Send(&data, 1, MPI_INT, i, 0, myFirstComm);
   }
}
